@@ -204,7 +204,7 @@ struct ServerOptions {
 
     // SSL related options. Refer to `ServerSSLOptions' for details
     bool has_ssl_options() const { return _ssl_options != NULL; }
-    const ServerSSLOptions& ssl_options() const { return *_ssl_options.get(); }
+    const ServerSSLOptions& ssl_options() const { return *_ssl_options; }
     ServerSSLOptions* mutable_ssl_options();
 
     // Whether the server uses rdma or not
@@ -547,6 +547,10 @@ public:
     int MaxConcurrencyOf(google::protobuf::Service* service,
                          const butil::StringPiece& method_name) const;
 
+    int Concurrency() const {
+        return butil::subtle::NoBarrier_Load(&_concurrency);
+    };
+
 private:
 friend class StatusService;
 friend class ProtobufsService;
@@ -696,6 +700,7 @@ friend class Controller;
     mutable bvar::Adder<int64_t> _nerror_bvar;
     mutable bvar::PerSecond<bvar::Adder<int64_t> > _eps_bvar;
     BAIDU_CACHELINE_ALIGNMENT mutable int32_t _concurrency;
+    bvar::PassiveStatus<int32_t> _concurrency_bvar;
 };
 
 // Get the data attached to current searching thread. The data is created by
